#!/usr/bin/env python3
from sysrepo_api import sysrepo_api3 as api
import hashlib
import subprocess
from shutil import copy2 as filecopy
import os
import syslog
import ftplib
from configobj import ConfigObj
import time

ZEBRAD_CONF_FILE = "zebra.conf"
OSPFD_CONF_FILE = "ospfd.conf"
OSPF6D_CONF_FILE = "ospf6d.conf"
BGPD_CONF_FILE = "bgpd.conf"

QUAGGA_CONF = '/etc/frr'
TMP_DIR = "/tmp"

STARTUP_SUCCESSFULL = '/run/sr_quagga_startup_ok'

serial_cfg_file = "/config/.serial-number"
try:
    serial=open(serial_cfg_file, "r").readline().strip()
except:
    serial="controller"


class SysrepoQuagga(api.SysrepoAPI):
    has_zebra_dl = True
    has_bgp_dl = True
    has_ospf_dl = True
    has_ospf6_dl = True
    mod_count = 0

    def __init__(self,
                 app_name="quagga",
                 log_file=None,
                 log_level=None,
                 console_log=False,
                 namespace="/ucpe:config/quagga:quagga",
                 run_as_subscriber=False,
                 priority=0):
        api.SysrepoAPI.__init__(
            self,
            app_name=app_name,
            log_file=log_file,
            log_level=log_level,
            console_log=console_log,
            namespace=namespace,
            run_as_subscriber=run_as_subscriber,
            priority=priority
        )
        

    def _validate_not_none(self, key_protocol, key_server, key_filename, key_filehashsum):
        retval = True

        if key_protocol is None or key_protocol is "":
            self.logger.error("Protocol is empty!")
            retval = False

        if key_server is None or key_server is "":
            self.logger.error("Server address is empty!")
            retval = False

        if key_filename is None or key_filename is "":
            self.logger.error("Filename is empty!")
            retval = False

        if key_filehashsum is None or key_filehashsum is "":
            self.logger.error("Hashsum is empty!")
            retval = False

        return retval

    def _calc_file_hashsum(self, key_filename, dir=TMP_DIR):
        try:
            filehashsum = hashlib.sha256()
            (mydir, filename) = os.path.split(key_filename)
            filehashsum.update(open('%s/%s' % (dir, filename), "rb").read())
            return filehashsum.hexdigest().upper(), filename
        except Exception as e:
            self.logger.debug("{}".format(e))
            return None, None

    def _delete_old_ip_addr(self, dst_dirname, dst_filename):
        retval = True
        conf_found = False
        curr_int = ''
        try:
            with open(dst_dirname + '/' + dst_filename, 'r') as f:
                for line in f.readlines():
                    if 'interface' in line:
                        conf_found = True
                        curr_int = str(line.split()[1])
                        continue
                    if conf_found:
                        if line.startswith('!'):
                            conf_found = False
                        else:
                            if 'ip address' in line:
                                try:
                                    ip_addr = str(line.split()[2])
                                    subprocess.check_call(['/usr/bin/vtysh', '-c', 'configure terminal', '-c',
                                                           'interface {}'.format(curr_int), '-c',
                                                           'no ip address {}'.format(ip_addr)])
                                    self.logger.info(
                                        "Deleted {} IP address from {} interface.".format(ip_addr, curr_int))
                                except Exception as e:
                                    retval = True
                                    self.logger.error("Failed to delete {} from configuration! {}".format(ip_addr, e))
        except Exception as e:
            retval = True
            self.logger.error("Configuration file {}/{} is not found! {}".format(dst_dirname, dst_filename, e))

        return retval

    def _copyfile(self, src_dirname, src_filename, dst_dirname, dst_filename):
        retval = False
        try:
            # Create directory if does not exist
            if not os.path.exists(dst_dirname):
                self.logger.info("Create directory %s!" % dst_dirname)
                os.makedirs(dst_dirname)

            try:
                # Copy file in /config/quagga
                self.logger.info("Copy file %s to %s." % (src_filename, dst_dirname))
                filecopy('%s/%s' % (src_dirname, src_filename), '%s/%s' % (dst_dirname, dst_filename))
                retval = True  # If we reach here all is ok

            except Exception as e:
                self.logger.error("Copy of file {} failed! {}".format(src_filename, e))
        except Exception as e:
            self.logger.error("Directory {} creation failed! {}".format(dst_dirname, e))

        return retval

    def _update_from_changeset(self, origvalue, change_set, service, item):
        retval = origvalue
        if change_set and service and item:
            for change in change_set:
                xpath = change['xpath']
                if xpath.find(service) > -1 and xpath.find(item) > -1:
                    if change["operation"] != 'DELETED':
                        retval = change['new_value']
                    break
        return retval

    def  clean_conf_files(self):
        if os.path.exists('/etc/frr/staticd.conf'):
            os.unlink('/etc/frr/staticd.conf')
        if os.path.exists('/etc/frr/zebra.conf'):
            filecopy('/etc/frr/zebra.conf', '/etc/frr/staticd.conf')

    def _apply_zebrad_config(self, change_set=None, service_enabled=False):
        retval = False
        self.logger.info("Zebra config file update routine started!")
        self.has_zebra_dl = True

        key_server = self.get_zebrad_config_param("config-server")
        key_protocol = self.get_zebrad_config_param("protocol")
        key_username = self.get_zebrad_config_param("username")
        key_password = self.get_zebrad_config_param("password")
        key_filename = self.get_zebrad_config_param("filename")
        key_filehashsum = self.get_zebrad_config_param("hash-sum")

        # Update from the current changeset
        if change_set is not None:
            key_server = self._update_from_changeset(key_server, change_set, "zebrad", "config-server")
            key_protocol = self._update_from_changeset(key_protocol, change_set, "zebrad", "protocol")
            key_username = self._update_from_changeset(key_username, change_set, "zebrad", "username")
            key_password = self._update_from_changeset(key_password, change_set, "zebrad", "password")
            key_filename = self._update_from_changeset(key_filename, change_set, "zebrad", "filename")
            key_filehashsum = self._update_from_changeset(key_filehashsum, change_set, "zebrad", "hash-sum")

        if service_enabled:
            if self._validate_not_none(key_protocol, key_server, key_filename, key_filehashsum):
                (filehashsum, filenameonly) = self._calc_file_hashsum(ZEBRAD_CONF_FILE, QUAGGA_CONF)
                if filehashsum and filehashsum == key_filehashsum.upper():
                    self.logger.info("Config file for zebrad is actual!")
                    retval = True  # If we reach here all is ok 
                else:
                    self.logger.info("Downloading of config file for zebrad from %s://%s!" % (key_protocol, key_server))
                    if quagga_file_get(key_protocol, key_server, key_username, key_password, key_filename, TMP_DIR):
                        (filehashsum, filenameonly) = self._calc_file_hashsum(key_filename)
                        if filehashsum and filehashsum == key_filehashsum.upper():
                            if self._delete_old_ip_addr(QUAGGA_CONF, ZEBRAD_CONF_FILE) and \
                               self._copyfile(TMP_DIR, filenameonly, QUAGGA_CONF, ZEBRAD_CONF_FILE):
                                retval = self._restart_service("zebrad")
                        else:
                            self.logger.error("Hashsum of config file zebrad-config does not mach!")
                    else:
                        self.logger.error("Downloading of config file for zebrad failed!")
                        self.has_zebra_dl = False
            else:
                self.logger.error("Invalid parameters received!")
        else:
            # If not enabled just return true, so sysrepo to can acceptdata
            retval = True

        return retval

    def _apply_ospfd_config(self, change_set=None, service_enabled=False):
        retval = False
        self.has_ospf_dl = True
        self.logger.info("OSPFD config file update routine started!")

        key_server = self.get_ospfd_config_param("config-server")
        key_protocol = self.get_ospfd_config_param("protocol")
        key_username = self.get_ospfd_config_param("username")
        key_password = self.get_ospfd_config_param("password")
        key_filename = self.get_ospfd_config_param("filename")
        key_filehashsum = self.get_ospfd_config_param("hash-sum")

        # Update from the current changeset
        if change_set is not None:
            key_server = self._update_from_changeset(key_server, change_set, "ospfd", "config-server")
            key_protocol = self._update_from_changeset(key_protocol, change_set, "ospfd", "protocol")
            key_username = self._update_from_changeset(key_username, change_set, "ospfd", "username")
            key_password = self._update_from_changeset(key_password, change_set, "ospfd", "password")
            key_filename = self._update_from_changeset(key_filename, change_set, "ospfd", "filename")
            key_filehashsum = self._update_from_changeset(key_filehashsum, change_set, "ospfd", "hash-sum")

        if service_enabled:
            if self._validate_not_none(key_protocol, key_server, key_filename, key_filehashsum):
                (filehashsum, filenameonly) = self._calc_file_hashsum(OSPFD_CONF_FILE, QUAGGA_CONF)
                if filehashsum and filehashsum == key_filehashsum.upper():
                    self.logger.info("Config file for ospfd is actual!")
                    retval = True  # If we reach here all is ok
                else:
                    self.logger.info("Downloading of config file for ospfd from %s://%s!" % (key_protocol, key_server))
                    if quagga_file_get(key_protocol, key_server, key_username, key_password, key_filename, TMP_DIR):
                        (filehashsum, filenameonly) = self._calc_file_hashsum(key_filename)
                        if filehashsum and filehashsum == key_filehashsum.upper():
                            if self._delete_old_ip_addr(QUAGGA_CONF, OSPFD_CONF_FILE) and \
                               self._copyfile(TMP_DIR, filenameonly, QUAGGA_CONF, OSPFD_CONF_FILE):
                                retval = self._restart_service("ospfd")
                        else:
                            self.logger.error("Hashsum of config file ospfd-config does not mach!")
                    else:
                        self.logger.error("Downloading of config file for ospfd failed!")
                        self.has_ospf_dl = False
            else:
                self.logger.error("Invalid parameters received!")
        else:
            # If not enabled just return true, so sysrepo to can acceptdata
            retval = True

        return retval

    def _apply_ospf6d_config(self, change_set=None, service_enabled=False):
        retval = False
        self.has_ospf6_dl = True
        self.logger.info("OSPF6D config file update routine started!")

        key_server = self.get_ospf6d_config_param("config-server")
        key_protocol = self.get_ospf6d_config_param("protocol")
        key_username = self.get_ospf6d_config_param("username")
        key_password = self.get_ospf6d_config_param("password")
        key_filename = self.get_ospf6d_config_param("filename")
        key_filehashsum = self.get_ospf6d_config_param("hash-sum")

        # Update from the current changeset
        if change_set is not None:
            key_server = self._update_from_changeset(key_server, change_set, "ospf6d", "config-server")
            key_protocol = self._update_from_changeset(key_protocol, change_set, "ospf6d", "protocol")
            key_username = self._update_from_changeset(key_username, change_set, "ospf6d", "username")
            key_password = self._update_from_changeset(key_password, change_set, "ospf6d", "password")
            key_filename = self._update_from_changeset(key_filename, change_set, "ospf6d", "filename")
            key_filehashsum = self._update_from_changeset(key_filehashsum, change_set, "ospf6d", "hash-sum")

        if service_enabled:
            if self._validate_not_none(key_protocol, key_server, key_filename, key_filehashsum):
                (filehashsum, filenameonly) = self._calc_file_hashsum(OSPF6D_CONF_FILE, QUAGGA_CONF)
                if filehashsum and filehashsum == key_filehashsum.upper():
                    self.logger.info("Config file for ospf6d is actual!")
                    retval = True  # If we reach here all is ok
                else:
                    self.logger.info("Downloading of config file for ospf6d from %s://%s!" % (key_protocol, key_server))
                    if quagga_file_get(key_protocol, key_server, key_username, key_password, key_filename, TMP_DIR):
                        (filehashsum, filenameonly) = self._calc_file_hashsum(key_filename)
                        if filehashsum and filehashsum == key_filehashsum.upper():
                            if self._copyfile(TMP_DIR, filenameonly, QUAGGA_CONF, OSPF6D_CONF_FILE):
                                retval = self._restart_service("ospf6d")
                        else:
                            self.logger.error("Hashsum of config file ospf6d-config does not mach!")
                    else:
                        self.logger.error("Downloading of config file for ospf6d failed!")
                        self.has_ospf6_dl = False
            else:
                self.logger.error("Invalid parameters received!")
        else:
            # If not enabled just return true, so sysrepo to can acceptdata
            retval = True

        return retval

    def _apply_bgpd_config(self, change_set=None, service_enabled=False):
        retval = False
        self.has_bgp_dl = True
        self.logger.info("BGPD config file update routine started!")

        key_server = self.get_bgpd_config_param("config-server")
        key_protocol = self.get_bgpd_config_param("protocol")
        key_username = self.get_bgpd_config_param("username")
        key_password = self.get_bgpd_config_param("password")
        key_filename = self.get_bgpd_config_param("filename")
        key_filehashsum = self.get_bgpd_config_param("hash-sum")

        # Update from the current changeset
        if change_set is not None:
            key_server = self._update_from_changeset(key_server, change_set, "bgpd", "config-server")
            key_protocol = self._update_from_changeset(key_protocol, change_set, "bgpd", "protocol")
            key_username = self._update_from_changeset(key_username, change_set, "bgpd", "username")
            key_password = self._update_from_changeset(key_password, change_set, "bgpd", "password")
            key_filename = self._update_from_changeset(key_filename, change_set, "bgpd", "filename")
            key_filehashsum = self._update_from_changeset(key_filehashsum, change_set, "bgpd", "hash-sum")

        if service_enabled:
            if self._validate_not_none(key_protocol, key_server, key_filename, key_filehashsum):
                (filehashsum, filenameonly) = self._calc_file_hashsum(BGPD_CONF_FILE, QUAGGA_CONF)
                if filehashsum and filehashsum == key_filehashsum.upper():
                    self.logger.info("Config file for bgpd is actual!")
                    retval = True  # If we reach here all is ok

                else:
                    self.logger.info("Downloading of config file for bgpd from %s://%s!" % (key_protocol, key_server))
                    if quagga_file_get(key_protocol, key_server, key_username, key_password, key_filename, TMP_DIR):
                        (filehashsum, filenameonly) = self._calc_file_hashsum(key_filename)
                        if filehashsum and filehashsum == key_filehashsum.upper():
                            if self._delete_old_ip_addr(QUAGGA_CONF, BGPD_CONF_FILE) and \
                               self._copyfile(TMP_DIR, filenameonly, QUAGGA_CONF, BGPD_CONF_FILE):
                                retval = self._restart_service("bgpd")
                        else:
                            self.logger.error("Hashsum of config file bgpd-config does not mach!")
                    else:
                        self.logger.error("Downloading of config file for bgpd failed!")
                        self.has_bgp_dl = False
            else:
                self.logger.error("Invalid parameters received!")
        else:
            # If not enabled just return true, so sysrepo to can acceptdata
            retval = True

        return retval

    def _start_stop_services(self, _start=False):
        retval = False
        if _start:
            start_stop_str = "yes"
        else:
            start_stop_str = "no"

        # Start/Stop
        try:
            cfg_obj = ConfigObjNet("/etc/frr/daemons")
            self.logger.info("%s ospfd.service" % ("Start" if _start else "Stop"))
            cfg_obj['ospfd'] = start_stop_str
            self.logger.info("%s ospf6d.service" % ("Start" if _start else "Stop"))
            cfg_obj['ospf6d'] = start_stop_str
            self.logger.info("%s bgpd.service" % ("Start" if _start else "Stop"))
            cfg_obj['bgpd'] = start_stop_str
            self.write_file_to_disk(cfg_obj)
            self.clean_conf_files()
            self._handle_frr_conf()
            subprocess.check_call(['systemctl', '--no-block', 'start', 'frr.service'])
            retval = True  # If we reach here all is ok
        except Exception as e:
            self.logger.error("Unable to {} service {}".format(start_stop_str, e))

        return retval
        
        
    def _handle_frr_conf(self):
        if os.path.exists('/etc/frr/zebra.conf') and os.path.getsize('/etc/frr/zebra.conf') > 0:
            filecopy('/etc/frr/zebra.conf', '/etc/frr/frr.conf')
        

    def _enable_disable_services(self, _enable=False):

        retval = False
        if _enable:
            enable_disable_str = "enable"
            enable_disable_str2 = "yes"
            enable_disable_str3 = "start"
        else:
            enable_disable_str = "disable"
            enable_disable_str2 = "no"
            enable_disable_str3 = "stop"

        try:
            # Enable/Disable
            cfg_obj = ConfigObjNet("/etc/frr/daemons")
            self.logger.info("%s ospfd.service" % ("Enable" if _enable else "Disable"))
            cfg_obj['ospfd'] = enable_disable_str2
            self.logger.info("%s ospf6d.service" % ("Enable" if _enable else "Disable"))
            cfg_obj['ospf6d'] = enable_disable_str2
            self.logger.info("%s bgpd.service" % ("Enable" if _enable else "Disable"))
            cfg_obj['bgpd'] = enable_disable_str2
            self.write_file_to_disk(cfg_obj)
            self.clean_conf_files()
            subprocess.check_call(['systemctl', enable_disable_str, 'frr.service'])
            subprocess.check_call(['systemctl', enable_disable_str3, 'frr.service'])
            retval = True  # If we reach here all is ok
        except Exception as e:
            self.logger.error("Unable to {} service {}".format(enable_disable_str, e))

        return retval

    def _enable_allservices(self, _start_enable=False):
        if _start_enable:
            self.logger.info("Routine for enable/disable of frr service started!")
            retval = self._enable_disable_services(_start_enable)
            if retval:
                self.logger.info("Routine for start/stop of frr service started!")
                retval = self._start_stop_services(_start_enable)
        else:
            self.logger.info("Routine for start/stop of frr service started!")
            retval = self._start_stop_services(_start_enable)
            if retval:
                self.logger.info("Routine for enable/disable of frr service started!")
                retval = self._enable_disable_services(_start_enable)

        return retval

    def _restart_service(self, _service):
        retval = False
        try:
            if _service in ["zebrad", "ospfd", "ospf6d", "bgpd"]:
                self.mod_count = self.mod_count + 1
            elif _service == "finally" and self.mod_count>0:
                self.mod_count = 0
                self.logger.info("Restart frr.service")
                self.clean_conf_files()
                self._handle_frr_conf()
                subprocess.check_call(['systemctl', '--no-block', 'restart', 'frr.service'])
            retval = True  # If we reach here all is ok
        except Exception as e:
            self.logger.error("Unable to restart frr.service {}".format(e))
            
        return retval

    def apply_config(self, change_set=None, apply_startup=False):
        if apply_startup:
            os.system("touch %s" % STARTUP_SUCCESSFULL)
            # Startup - Get enable flag
            if self.get_enabled():
                retval = self._enable_allservices(True)
                # Not every time the configuration is updated, only on enabled, so we must update it here!
                retval &= self._apply_zebrad_config(change_set, True)
                retval &= self._apply_ospfd_config(change_set, True)
                retval &= self._apply_ospf6d_config(change_set, True)
                retval &= self._apply_bgpd_config(change_set, True)
                self._restart_service("finally")
                #if we failed to download some of the configs - touch them so that we can signal manager that config file is different.
                if not self.has_zebra_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, ZEBRAD_CONF_FILE))
                    os.system("rm -f  %s 2>/dev/null 1>/dev/null" % STARTUP_SUCCESSFULL)
                if not self.has_bgp_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, BGPD_CONF_FILE))
                    os.system("rm -f  %s 2>/dev/null 1>/dev/null" % STARTUP_SUCCESSFULL)
                if not self.has_ospf_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, OSPFD_CONF_FILE))
                    os.system("rm -f  %s 2>/dev/null 1>/dev/null" % STARTUP_SUCCESSFULL)
                if not self.has_ospf6_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, OSPF6D_CONF_FILE))
                    os.system("rm -f  %s 2>/dev/null 1>/dev/null" % STARTUP_SUCCESSFULL)
            else:
                retval = self._enable_allservices(False)
        else:
            # Flow flags
            zebrad_config = False
            ospfd_config = False
            ospf6d_config = False
            bgpd_config = False
            os.system("touch %s" % STARTUP_SUCCESSFULL)

            enable_quagga_services = self.get_enabled()

            # Xpath change - iterate changes and set corresponding flags
            for idx in range(0, len(change_set)):
                if not change_set[idx]["xpath"].startswith(self.namespace):
                    continue
                if change_set[idx]["operation"] == 'MODIFIED' or \
                   change_set[idx]["operation"] == 'CREATED':
                    # check the xpath
                    if "enabled" in change_set[idx]["xpath"]:
                        if str(change_set[idx]["new_value"]).lower() == "true":
                            enable_quagga_services = True
                        else:
                            enable_quagga_services = False
                    elif "zebrad-config" in change_set[idx]["xpath"]:
                        zebrad_config = True
                    elif "ospfd-config" in change_set[idx]["xpath"]:
                        ospfd_config = True
                    elif "ospf6d-config" in change_set[idx]["xpath"]:
                        ospf6d_config = True
                    elif "bgpd-config" in change_set[idx]["xpath"]:
                        bgpd_config = True

            # Execute flow. On that way, the config should be called only once despite how many times is called
            retval = self._enable_allservices(enable_quagga_services)

            if zebrad_config:
                retval &= self._apply_zebrad_config(change_set, enable_quagga_services)

            if ospfd_config:
                retval &= self._apply_ospfd_config(change_set, enable_quagga_services)

            if ospf6d_config:
                retval &= self._apply_ospf6d_config(change_set, enable_quagga_services)

            if bgpd_config:
                retval &= self._apply_bgpd_config(change_set, enable_quagga_services)
            if enable_quagga_services:
                self._restart_service("finally")
                
            if enable_quagga_services:
                #if we failed to download some of the configs - touch them so that we can signal manager that config file is different.
                if not self.has_zebra_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, ZEBRAD_CONF_FILE))
                if not self.has_bgp_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, BGPD_CONF_FILE))
                if not self.has_ospf_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, OSPFD_CONF_FILE))
                if not self.has_ospf6_dl:
                    os.system("touch %s/%s" % ( QUAGGA_CONF, OSPF6D_CONF_FILE))

        return True

    def set_enabled(self, enabled=False):
        self.set_item("{}/enabled".format(self.namespace), enabled)

    def get_enabled(self):
        return self.get_item("{}/enabled".format(self.namespace))

    def unset_enabled(self):
        self.delete_item("{}/enabled".format(self.namespace))

    def set_zebrad_config_param(self, param, value):
        self.set_item("{}/zebrad-config/{}".format(self.namespace, param), value)

    def get_zebrad_config_param(self, param):
        return self.get_item("{}/zebrad-config/{}".format(self.namespace, param))

    def unset_zebrad_config_param(self, param):
        self.delete_item("{}/zebrad-config/{}".format(self.namespace, param))

    def set_ospfd_config_param(self, param, value):
        self.set_item("{}/ospfd-config/{}".format(self.namespace, param), value)

    def get_ospfd_config_param(self, param):
        return self.get_item("{}/ospfd-config/{}".format(self.namespace, param))

    def unset_ospfd_config_param(self, param):
        self.delete_item("{}/ospfd-config/{}".format(self.namespace, param))

    def set_ospf6d_config_param(self, param, value):
        self.set_item("{}/ospf6d-config/{}".format(self.namespace, param), value)

    def get_ospf6d_config_param(self, param):
        return self.get_item("{}/ospf6d-config/{}".format(self.namespace, param))

    def unset_ospf6d_config_param(self, param):
        self.delete_item("{}/ospf6d-config/{}".format(self.namespace, param))

    def set_bgpd_config_param(self, param, value):
        self.set_item("{}/bgpd-config/{}".format(self.namespace, param), value)

    def get_bgpd_config_param(self, param):
        return self.get_item("{}/bgpd-config/{}".format(self.namespace, param))

    def unset_bgpd_config_param(self, param):
        self.delete_item("{}/bgpd-config/{}".format(self.namespace, param))


#
def quagga_sftp_file_get(address, user, pswd, srv_path, local_path):
    syslog.syslog(syslog.LOG_DEBUG, 'Going sftp://%s/%s to %s' % (address, srv_path, local_path))

    try:
        if pswd is None:
            subprocess.check_call(['sshpass', '-p', '', 'sftp', '-oStrictHostKeyChecking=no', '-oCheckHostIP=no',
                                   '%s@%s:%s %s' % (user, address, srv_path, local_path)])
        else:
            subprocess.check_call(['sshpass', '-p', pswd, 'sftp', '-oStrictHostKeyChecking=no', '-oCheckHostIP=no',
                                   '%s@%s:%s %s' % (user, address, srv_path, local_path)])
    except subprocess.CalledProcessError as e:
        syslog.syslog(syslog.LOG_ERR, '\"%s\" failed with code %d' % (e.cmd, e.returncode))
        return False
    except OSError as e:
        syslog.syslog(syslog.LOG_ERR, 'Fail - %s' % e.strerror)
        return False

    return True
    
    
def quagga_https_file_get(address, user, pswd, srv_path, local_path):
    syslog.syslog(syslog.LOG_DEBUG, 'Going https://%s/%s to %s' % (address, srv_path, local_path))
    
    port = 30636
    (thedir, thefilename) = os.path.split(srv_path)
    syslog.syslog(syslog.LOG_DEBUG, 'Going https://%s:%d/%s to %s' % (address, port, srv_path, local_path))
        
    try:
            retcode = os.system('curl -s -k -X GET https://%s:%d/%s -H \'Authentication: %s\' -H \'accept: application/binary\' -o %s/%s' 
                        % (address, port, srv_path,serial, local_path,thefilename))
            if retcode != 0:
                syslog.syslog(syslog.LOG_ERR, "failed with code %d" % (retcode))
                return False
    except :
        syslog.syslog(syslog.LOG_ERR, 'Fail - %s' % e.strerror)
        return False

    return True


def quagga_ftp_file_get(address, user, pswd, srv_path, local_path):
    
    # Get path and file name
    if srv_path.startswith("/inventory"):
        srv_path = srv_path.replace("/inventory","",1)

    syslog.syslog(syslog.LOG_DEBUG, 'Going ftp://%s/%s to %s' % (address, srv_path, local_path))
    try:
        ftp = ftplib.FTP(host=address, user=user, passwd=pswd, timeout=15)

        d_name = os.path.dirname(srv_path)
        f_name = os.path.basename(srv_path)

        # if '' != d_name:
        #    ftp.sendcmd( 'CD %s' % d_name )

        loc_file = open('%s/%s' % (local_path, f_name), 'wb')
        ftp.retrbinary('RETR %s' % srv_path, loc_file.write)
        # ftp.retrbinary('RETR %s' % f_name, loc_file.write)
        loc_file.close()
    except ftplib.all_errors as e:
        syslog.syslog(syslog.LOG_ERR, 'FTP download failed ! -- \'%s\'' % e)
        return False

    return True


def quagga_file_get(proto, address, user, pswd, srv_path, local_path):
    syslog.syslog(syslog.LOG_DEBUG, 'quagga_file_get -> %s://%s@%s' % (proto, user, address))
    if None == srv_path:
        syslog.syslog(syslog.LOG_ERR, 'File for download is not specifid')
        return False

    loop=4
    dl_ok = False
    while loop > 0:
        loop -= 1

        if 'sftp' == proto:
            dl_ok = quagga_sftp_file_get(address, user, pswd, srv_path, local_path)
        elif 'ftp' == proto:
            dl_ok =  quagga_ftp_file_get(address, user, pswd, srv_path, local_path)
        elif 'https' == proto:
            dl_ok =  quagga_https_file_get(address, user, pswd, srv_path, local_path)
        else:
            syslog.syslog(syslog.LOG_ERR, 'Protocol %s is not supported!' % proto)
            return False
        if dl_ok:
            return True
        time.sleep(15)


class ConfigObjNet( ConfigObj ):
    def _write_line(self, indent_string, entry, this_entry, comment):
        """Write an individual line, for the write method"""
        # NOTE: the calls to self._quote here handles non-StringType values.
        if not self.unrepr:
            val = self._decode_element(self._quote(this_entry))
        else:
            val = repr(this_entry)

        return '%s%s%s%s%s' % (indent_string,
                               self._decode_element(self._quote(entry, multiline=False)),
                               self._a_to_u('='),
                               val,
                               self._decode_element(comment))

